//-----------------------------------------------------
// RFM12 Ansteuerung
//
// erstellt von Michael Walischewski
//          am: 05.02.2011
//
// Hardwareschicht zur Ansteuerung des RFM12
//
//
//-----------------------------------------------------

#include "../Utils/RFM12.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <util/delay.h>
#include "../HomeReceiver/global.h"

#include <inttypes.h>

#if defined(__AVR_ATtiny85__)
void portInit() {

    SPIDDR = (1<<CS) | (1<<SDI) | (1<<SCK);
    SPIDDR &= ~(1<<SDO);
    DDRB  &= ~(1<<NIRQ);

    HI(CS);
    HI(SDO);
    LO(SCK);

    MCUCR = (1<<ISC01);                          // fallende Flanke INT0
    GIMSK |= (1<<INT0);
}

#elif defined(__AVR_ATmega128__)
void portInit() {

    SPIDDR = (1<<CS) | (1<<SDI) | (1<<SCK);
    SPIDDR &= ~(1<<SDO);
    DDRD  &= ~(1<<NIRQ);
    PORTD |= (1<<NIRQ);

    HI(CS);
    HI(SDO);
    LO(SCK);

    EICRA |= (1<<ISC01);                          // fallende Flanke INT0

    EIMSK |= (1<<INT0);


}
#else
#  error "Unknown MCU type."
#endif




unsigned int writeCmd(unsigned int cmd) {
  unsigned char i;
    unsigned int recv;
    recv = 0;
    LO(SCK);
    LO(CS);
    for(i=0; i<16; i++) {
      if(cmd&0x8000) HI(SDI); else LO(SDI);
      HI(SCK);
      recv<<=1;
      if( SPIPIN&(1<<SDO) ) {
        recv|=0x0001;
      }
      LO(SCK);
      cmd<<=1;
    }
    HI(CS);
    return recv;
}



unsigned int rf12_ready(void){
	LO(CS);
	asm("nop");
	asm("nop");
	uint32_t timeout = 0;

	while ((!(SPIPIN&(1<<SDO)))|| (++timeout> 5000000)); // entweder FIFO oder timeout

	HI(CS);
	if(timeout >5000000){
	    clearFIFO();
	    return 0;
	}
	else{
	 return 1;
	}
}



void rfInit() {

    writeCmd(0xC0E0);
	writeCmd(0x80E7);

	writeCmd(0xC2AB);
	writeCmd(0xCA81);
	writeCmd(0xE000);
	writeCmd(0xC800);
	writeCmd(0xC4B7);
	writeCmd(0xA640);
	writeCmd(0x9800);
	writeCmd(0x9589);
    writeCmd(0xC623);                   // 9600 Baud
    writeCmd(0xCA83);
	writeCmd(0x0000);
}

void switch2TX(void){
   writeCmd(0x0000);
   writeCmd(0x8239);
   _delay_ms(1);
}

void switch2RX(void){
    writeCmd(0x82D9);
    _delay_ms(1);
    clearFIFO();
}

void clearFIFO(void){
    writeCmd(0xCA81);
    _delay_ms(2);
    writeCmd(0xCA83);
    writeCmd(0x0000);
}


void rfSend(unsigned char data){
    if(rf12_ready()==1){
    writeCmd(0xB800 + data);}

}

unsigned char rfReceive(){
    unsigned int D=0;

    if(rf12_ready()==1){
    D=writeCmd(0XB000);
    return(D&0x00FF);
    }
    else return 0;
}

unsigned char rfm12Batt()
{
  unsigned char i;
  unsigned char status;

  writeCmd(0x8204);                          // Low Batt Detektor einschalten

  for (i=4; i<30; i++){                       // Startwert 2.6V  in 0.1 Schritte
      writeCmd(0xC0E0 | i);                  // Schwellwert setzen
      status = (writeCmd(0x0000)>>8);        // Status lesen
      if (status & (1 << 2))                  // LBD bit im Status 1?
          break;                            // wir haben den Batterie Wert !
  }
  writeCmd(0xC0E0); 					     // LowBatt wieder abschalten
  return i;
}



// Empfang-Signalisierung ber INT0
// klein und fein
ISR(INT0_vect)
{

   uint8_t temp  = (writeCmd(0x0000)>>8);
   if (temp & (1 << 7)){ //FFIT Interrupt
      INT_nIRQ=1;
   }

}
