//----------------------------------------------------------
// Protokoll fr RFM12
//
// erstellt am 08.03.2011 Michael Walischewski
//
//  Paket Message :
//
// [0]   - Sender ID
// [1]   - Empfnger ID
// [2]   - Pakettyp : 01 - Temp,
//                    02 - Adresspaket ,
//                    03 - rel. Luftfeuchte ,
//                    04 - ACK,
//                    05 - Adressen ACK,
//                    06 - Batteriewert,
//                    07 - schalten.
// [3]   - Anzahl Datenbytes
// [4+n] - Datenbytes
// [5+n] - CRC1
// [6+n] - CRC2 (16bit)
//
//
//----------------------------------------------------------


#include <stdlib.h>
#include "../Utils/RFM12.h"
#include "avr/interrupt.h"
#include <util/delay.h>
#include "../HomeReceiver/global.h"

#if defined(__AVR_ATmega128__)
   #include "../Utils/uart128.h"
#endif
#include <util/crc16.h>

volatile uint16_t Timer  = 0;
#if defined(__AVR_ATtiny85__)
  volatile uint16_t Timer8 = 0;
#endif

//--------------------------------------------------------
// Timer initialisieren fr Timeout bzw. Receive ACK
//--------------------------------------------------------
void TimerOn(void){

#if defined(__AVR_ATtiny85__)
   TCCR0A  =  (1<<WGM01);                   // CTC - Mode
   TCCR0B  =  (1<<CS02)| (1<<CS00) ;        // Prescaler 1/1024 / Clock = 1/8 CPU-Takt   doc S  83
   OCR0A   =  0xFF;
   TIMSK   =  (1<<OCIE0A);
   Timer8  = 0;
#elif defined(__AVR_ATmega128__)
   TCCR1B = (1<<WGM12) | (1<<CS12)| (1<<CS10);// Prescaler 128
   TCNT1  = 0;   // initialisieren mit 0
   OCR1A  = 15600;  // 1 sek. Takt bei 16Mhz
   TCCR1A &= ~ ((1 << COM1A0) | (1 << COM1A1));
   TIMSK  |= (1<<OCIE1A);
#else
#  error "Unknown MCU type."
#endif
   Timer=0;
}

void TimerOff(void){
#if defined(__AVR_ATtiny85__)
  TIMSK  &= ~(1<<OCIE0A);
  Timer8  =0;
#elif defined(__AVR_ATmega128__)
  TIMSK  &= ~(1<<OCIE1A);
#else
#  error "Unknown MCU type."
#endif
  Timer=0;
}

//--------------------------------------------------------
// Timer Interrupt verarbeiten
//--------------------------------------------------------

#if defined(__AVR_ATtiny85__)
ISR(TIMER0_COMPA_vect){

  Timer8++;
  if(Timer8 > 30){
      Timer++;
      Timer8=0;

  }

  // berlauf vermeiden
  if (Timer > 255)
      Timer = 0;

}
#elif defined(__AVR_ATmega128__)
 ISR(TIMER1_COMPA_vect){

  Timer++;
  // berlauf vermeiden
  if (Timer > 255)
      Timer = 0;

}
#else
#  error "Unknown MCU type."
#endif


uint16_t calcCRC(unsigned char *RFM12Message){

    uint16_t crc = 0xffff;
    int i = 0;


    for(i=0; i < RFM12Message[3]+4; i++){
        crc = _crc16_update(crc,RFM12Message[i]);
    }

    return crc;
}

//----------------------------------------------------------
// Datenpaket senden
// 6 Byte + CRC
//----------------------------------------------------------

unsigned char sendFrame(unsigned char *RFM12Message){

   uint16_t crc2=0;
   int i=0;
   unsigned char a,b;


   crc2 = calcCRC(RFM12Message);

    // Paket senden und anschlieen CRC anhngen !

    rfSend(0xAA); // PREAMBLE
    rfSend(0xAA);
    rfSend(0xAA);
    rfSend(0x2D); // SYNC
    rfSend(0xD4);

    for(i=0; i<RFM12Message[3]+4; i++){
       rfSend(RFM12Message[i]);
    }

    a=(crc2>>8) & 0xFF;
    RFM12Message[i+1]=a;
    rfSend(a);
    b=crc2 & 0xFF;
    RFM12Message[i+2]=b;
    rfSend(b);

    // Hat sich bewhrt !
    rfSend(0x0);//Dummy
    rfSend(0x0);

    return 0;
}

//----------------------------------------------------
// Es wurde ber den INT0 einen Empfang besttigt
// Jetzt pollen wir und hollen alles ab was da ist.
// geprft wird der Empfnger und CRC
//----------------------------------------------------

unsigned char Receive(unsigned char *RFM12Message){

    int i = 0;
    uint16_t crc2;
    unsigned char a=0,b=0;
    uint8_t Status = 0;

    if(INT_nIRQ !=0){

    RFM12Message[0] = rfReceive();   // QuellAdresse
    RFM12Message[1] = rfReceive();   // Zieladresse

    // hier direkt prfen, ob das Paket fr uns ist bzw. an alle !

    if((RFM12Message[1]==ID)||(RFM12Message[1]==0xFF)){

       RFM12Message[2] = rfReceive();   // Messagetyp
       RFM12Message[3] = rfReceive();   // Anzahlbytes

       for(i=0;i< RFM12Message[3]+2 ;i++){   // neben den Datenbyte auch noch die CRC Bytes +2 !
          RFM12Message[4+i]=rfReceive();
       }

       crc2 = calcCRC(RFM12Message);

       a    = (crc2>>8) & 0xFF;
       b    = crc2 & 0xFF;

       if ((a==RFM12Message[i+2]) && (b==RFM12Message[i+3])){
          if((RFM12Message[2]==0x02)){
            Status = 2;  // Adresse !
          }
          if(RFM12Message[2]==0x04){
            Status = 4;  // ACK !
          }
          if(RFM12Message[2]==0x01){
            Status = 1;  // Sensordaten
          }
          if(RFM12Message[2]==0x06){
            Status = 6;  // Batterie
          }
          if((RFM12Message[1]=0xFF) && (RFM12Message[2]==0x05) && (RFM12Message[4]==UNIQUE[0]) && (RFM12Message[5]==UNIQUE[1]) && (RFM12Message[6]==UNIQUE[2]) && (RFM12Message[7]==UNIQUE[3]) && (RFM12Message[8]==UNIQUE[4]) && (RFM12Message[9]==UNIQUE[5]) && (RFM12Message[10]==UNIQUE[6]) && (RFM12Message[11]==UNIQUE[7]) ){
            Status = 5;  // Adressen ACK !
          }
       }
       else{
         Status = 99;  // CRC falsch
       }
    }
    else {
       Status = 0; // Message ist nicht fr uns
    }
    _delay_ms(1);
    clearFIFO();
    }



    return Status;

}

//---------------------------------------------
// acknowledge Paket versenden
//---------------------------------------------

unsigned char sendACK(unsigned char *ACKPacket, unsigned char *RFM12Message){

   int i =0;

       ACKPacket[0] = ID;
       ACKPacket[1] = RFM12Message[0]; // An den Master

       switch2TX();

       for (i=0;i<1;i++){
          sendFrame(ACKPacket);
       }

       switch2RX();  // in EMpfangsmodus schalten

   return 1;
}

//-------------------------------------------------------------
// Frame senden und auf Antwort warten
// 3 Versuche , danach abbruch
//-------------------------------------------------------------
unsigned char sendPackage(unsigned char *RFM12Message){

    uint8_t Try =0;
    unsigned char timeout = 0;
   // char buffer[20];

    for(Try=1;Try < 4; Try++){
      switch2TX();    // TX ON
      sendFrame(RFM12Message);    //
      writeCmd(0x8208);  //tx off

      switch2RX();
      INT_nIRQ=0;
      TimerOn();          // fuer den Timeout
      while ((Receive(RFM12Message) != 4) && (Timer< 4));  //Wenn kein ACK und Timer ber 200 dann weiter
     if ((Timer >= 4) && (Try == 4)){
        timeout = 1;  // 3. Versuch und timeout , hier lief was schief !
      }
      if (Timer <4){
         // wir haben einen ACK erhalten !
         TimerOff();
         return 0;
      }
      TimerOff();
    } // for try
    return timeout;
}


//----------------------------------------------------
// ID vom Master anfordern
//----------------------------------------------------

unsigned char getID(unsigned char *RFM12Message){
   // Paket aufbauen fr den Master
   int i =0;

   RFM12Message[0] = 0xFF;   // Paket als Broadcast definieren
   RFM12Message[1] = 0x01;   // fuer den Master bestimmt
   RFM12Message[2] = 0x02;   // 01 - Temp, 02 - Adresspaket , 03 - rel. Luftfeuchte , 04 - ACK, 05 - Adressen ACK, 06 - schalten, usw. mehr habe ich bis jetzt
   RFM12Message[3] = 0x08;   // 8 Bytes - DIE UNIQUE

   for(i=0;i<8;i++){
      RFM12Message[i+4] = UNIQUE[i];   // boah bin ich faul
   }
   // und diese Unique Nummer schicken wir auf Reise , hnlich dem MAC-Prinzip, gefllt mir bis jetzt
   switch2TX();    // TX ON


   if(sendFrame(RFM12Message)==0){
      writeCmd(0x8208);  //tx off
      switch2RX();
      // hier bleiben wir so lange , bis wir unsere ID haben, ansonsten macht das Senden keinen Sinn !
      while(Receive(RFM12Message)!=5);
   }

   return RFM12Message[12];

}

unsigned char sendID(unsigned char *RFM12Message,unsigned char ID){

   uint16_t timeout = 0;
   RFM12Message[0] = 0x01;
   RFM12Message[1] = 0xFF;
   RFM12Message[2] = 0x05; // Adressen ACK
   RFM12Message[3] = 0x09; // 8 Bytes - DIE UNIQUE + Adresse

   // Rest bleibt wie empfangen !
   RFM12Message[12]=ID;
   switch2TX();    // TX ON
   if(sendFrame(RFM12Message)==0){
      writeCmd(0x8208);  //tx off
      switch2RX();
      INT_nIRQ=0;
      while((Receive(RFM12Message)!=4) && (++timeout < 20));
   }
   return 1;
}

unsigned char sendBatt(unsigned char *RFM12Message, unsigned char ID){

   unsigned char BattH, BattL,rslt;

   BattH = rfm12Batt()+26;      // offset erhalten
   BattL = BattH%10;
   BattH = BattH/10;


   RFM12Message[0]=ID;
   RFM12Message[1]=0x01;       // An den Master
   RFM12Message[2]=0x06;     // Messagetyp , hier Batteriespannung
   RFM12Message[3]=0x02;        // Anzahl Datenbytes
   RFM12Message[4]=BattH;
   RFM12Message[5]=BattL;

   rslt = sendPackage(RFM12Message);

   return rslt;
}

void RFM12init(){
    _delay_ms(20);
   rfInit();
   _delay_ms(20);
   switch2RX();
}




